"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const gtfs = require("gtfs-stream");
/**
 * Returns trips, transfers, interchange time and calendars from a GTFS zip.
 */
class GTFSLoader {
    constructor(gtfsStream) {
        this.gtfsStream = gtfsStream;
    }
    /**
     * Create a mutable loader then add each row until the end of the stream is reached
     */
    load(file, stopPrefix = "", filterBefore) {
        return new Promise(resolve => {
            const handler = new MutableGTFSLoader(stopPrefix, filterBefore);
            file.pipe(this.gtfsStream())
                .on("data", entity => handler.process(entity))
                .on("end", () => resolve(handler.getResults()));
        });
    }
}
exports.GTFSLoader = GTFSLoader;
/**
 * Encapsulation of mutable state while the results are loaded from the zip
 */
class MutableGTFSLoader {
    constructor(stopPrefix = "", filterBefore) {
        this.stopPrefix = stopPrefix;
        this.filterBefore = filterBefore;
        this.transfers = {};
        this.result = {
            trips: [],
            transfers: [],
            calendars: [],
            calendarDates: {},
            stopTimes: [],
            routes: [],
            agencies: [],
            stops: [],
            parentStops: {}
        };
    }
    process(entity) {
        return this[entity.type] && this[entity.type](entity.data);
    }
    /**
     * Flatten the transfers and return the results
     */
    getResults() {
        return {
            ...this.result,
            transfers: Object.values(this.transfers)
        };
    }
    trip(row) {
        this.result.trips.push(row);
    }
    stop_time(row) {
        if (row.departure_time && row.arrival_time) {
            row.stop_id = this.stopPrefix + row.stop_id;
            this.result.stopTimes.push(row);
        }
    }
    route(row) {
        this.result.routes.push(row);
    }
    stop(row) {
        if (!row.parent_station) {
            row.stop_id = this.stopPrefix + row.stop_id;
            row.stop_lon = +row.stop_lon;
            row.stop_lat = +row.stop_lat;
            this.result.stops.push(row);
        }
        else {
            this.result.parentStops[row.stop_id] = row.parent_station;
        }
    }
    agency(row) {
        this.result.agencies.push(row);
    }
    calendar(row) {
        const inPast = !this.filterBefore || row.end_date < this.filterBefore;
        const runs = row.monday
            || row.tuesday
            || row.wednesday
            || row.thursday
            || row.friday
            || row.saturday
            || row.sunday;
        if (!inPast && runs) {
            this.result.calendars.push(row);
        }
    }
    calendar_date(row) {
        if (!this.filterBefore || row.date >= this.filterBefore) {
            this.result.calendarDates[row.service_id] = this.result.calendarDates[row.service_id] || [];
            this.result.calendarDates[row.service_id].push(row);
        }
    }
    link(row) {
        row.min_transfer_time = row.duration;
        row.transfer_type = 2;
        this.transfer(row);
    }
    transfer(row) {
        row.from_stop_id = this.stopPrefix + row.from_stop_id;
        row.to_stop_id = this.stopPrefix + row.to_stop_id;
        const key = row.from_stop_id + "_" + row.to_stop_id;
        if (!this.transfers[key] || this.transfers[key].min_transfer_time > row.min_transfer_time) {
            this.transfers[key] = row;
        }
    }
}
exports.gtfsStreamFactory = () => gtfs({ raw: true });
