"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const toGTFSDate_1 = require("./toGTFSDate");
/**
 * Creates calendars based on a set of calendar dates.
 */
class CalendarFactory {
    /**
     * Count the number of times a service runs or does not run on each day of the week and then decide whether to
     * create a calendar with that day enabled with exclude days when it is not running, or disabled with include
     * days when it is running.
     */
    create(serviceId, calendarDates) {
        calendarDates.sort((a, b) => +a.date - +b.date);
        const startDate = calendarDates[0];
        const endDate = calendarDates[calendarDates.length - 1];
        const [daysRunning, daysNotRunning] = this.splitCalendarDates(calendarDates, serviceId, startDate, endDate);
        const calendar = this.createCalendar(serviceId, startDate, endDate, daysRunning, daysNotRunning);
        const newCalendarDates = this.getCalendarDates(daysRunning, daysNotRunning);
        return [calendar, newCalendarDates];
    }
    splitCalendarDates(calendarDates, serviceId, startDate, endDate) {
        const daysNotRunning = [[], [], [], [], [], [], []];
        const daysRunning = [[], [], [], [], [], [], []];
        const calendarDateIndex = this.indexCalendarDates(calendarDates);
        let i = startDate.date;
        let date = this.getDateFromGTFSString(i);
        while (i <= endDate.date) {
            const dow = date.getDay();
            if (calendarDateIndex[i]) {
                daysRunning[dow].push({ exception_type: 1, service_id: serviceId, date: i });
            }
            else {
                daysNotRunning[dow].push({ exception_type: 2, service_id: serviceId, date: i });
            }
            date.setDate(date.getDate() + 1);
            i = toGTFSDate_1.toGTFSDate(date);
        }
        return [daysRunning, daysNotRunning];
    }
    indexCalendarDates(calendarDates) {
        return calendarDates.reduce((index, calendarDate) => {
            index[calendarDate.date] = calendarDate;
            return index;
        }, {});
    }
    getDateFromGTFSString(i) {
        return new Date(i.substr(0, 4) + "-" + i.substr(4, 2) + "-" + i.substr(6, 2));
    }
    createCalendar(serviceId, startDate, endDate, daysRunning, daysNotRunning) {
        return {
            service_id: serviceId,
            start_date: startDate.date,
            end_date: endDate.date,
            sunday: daysRunning[0].length > daysNotRunning[0].length ? 1 : 0,
            monday: daysRunning[1].length > daysNotRunning[1].length ? 1 : 0,
            tuesday: daysRunning[2].length > daysNotRunning[2].length ? 1 : 0,
            wednesday: daysRunning[3].length > daysNotRunning[3].length ? 1 : 0,
            thursday: daysRunning[4].length > daysNotRunning[4].length ? 1 : 0,
            friday: daysRunning[5].length > daysNotRunning[5].length ? 1 : 0,
            saturday: daysRunning[6].length > daysNotRunning[6].length ? 1 : 0
        };
    }
    /**
     * For each day of the week check if the service runs more often than not. If it does, return the exclude days as
     * the calendar will have the day set to 1, if not then return the include days.
     */
    getCalendarDates(daysRunning, daysNotRunning) {
        return daysRunning.flatMap((runningDates, i) => {
            return runningDates.length > daysNotRunning[i].length ? daysNotRunning[i] : runningDates;
        });
    }
}
exports.CalendarFactory = CalendarFactory;
